# Generated by Django 2.1.5 on 2019-01-28 22:30

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0004_auto_20190128_1349'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authtoken',
            name='token',
            field=models.CharField(db_index=True, max_length=138, unique=True, validators=[django.core.validators.RegexValidator('^[-a-zA-Z0-9_/]+\\Z', 'Enter a valid token.', 'invalid')]),
        ),
    ]
