# Generated by Django 2.1.5 on 2019-02-11 23:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0006_auto_20190128_2251'),
    ]

    operations = [
        migrations.AddField(
            model_name='usercomponent',
            name='allow_domain_mode',
            field=models.BooleanField(default=False),
        ),
    ]
