# Generated by Django 2.1.7 on 2019-03-31 12:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0010_assignedcontent_attached_to_primary_anchor'),
    ]

    operations = [
        migrations.AddField(
            model_name='usercomponent',
            name='default_content_features',
            field=models.ManyToManyField(blank=True, help_text='Select features used by default for contents in this component', limit_choices_to=models.Q(ctype__contains='g'), related_name='default_feature_for_contents', to='spider_base.ContentVariant'),
        ),
        migrations.AlterField(
            model_name='usercomponent',
            name='primary_anchor',
            field=models.ForeignKey(blank=True, help_text='Select main identifying anchor. Also used for attaching persisting tokens (elsewise they are attached to component)', limit_choices_to={'info__contains': '\nanchor\n'}, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='primary_anchor_for', to='spider_base.AssignedContent'),
        ),
    ]
