# Generated by Django 2.1.5 on 2019-01-28 13:39

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0002_20190127_squashed'),
    ]

    operations = [
        migrations.AddField(
            model_name='assignedcontent',
            name='token',
            field=models.CharField(blank=True, db_index=True, max_length=138, null=True, unique=True, validators=[django.core.validators.RegexValidator('^[-a-zA-Z0-9_/]+\\Z', 'Enter a valid token.', 'invalid')]),
        ),
        migrations.AddField(
            model_name='usercomponent',
            name='token',
            field=models.CharField(blank=True, db_index=True, max_length=138, null=True, unique=True, validators=[django.core.validators.RegexValidator('^[-a-zA-Z0-9_/]+\\Z', 'Enter a valid token.', 'invalid')]),
        ),
        migrations.AlterField(
            model_name='assignedcontent',
            name='nonce',
            field=models.SlugField(blank=True, db_index=False, max_length=120, null=True),
        ),
        migrations.AlterField(
            model_name='authtoken',
            name='token',
            field=models.CharField(db_index=True, max_length=137, unique=True, validators=[django.core.validators.RegexValidator('^[-a-zA-Z0-9_/]+\\Z', 'Enter a valid token.', 'invalid')]),
        ),
        migrations.AlterField(
            model_name='usercomponent',
            name='nonce',
            field=models.SlugField(blank=True, db_index=False, max_length=120, null=True),
        ),
    ]
