# Generated by Django 2.1 on 2018-08-10 14:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_filets', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='textfilet',
            name='non_public_edit',
            field=models.BooleanField(default=False, help_text='Allow others to edit text file if not public'),
        ),
    ]
