# Generated by Django 2.1 on 2018-08-15 11:28

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('spider_tags', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='taglayout',
            name='default_verifiers',
            field=jsonfield.fields.JSONField(blank=True, default=list),
        ),
    ]
