{% load spider_rdf %}
{% include "spider_base/partials/form_errors.html" with form=form %}
<{% if not inner_form %}form action="{{ request.get_full_path }}" enctype="{{enctype}}" method="post"{% else %}div{% endif %}>
  {% for rdftype in add_spkc_types %}
    <data property="spkc:type" hidden="hidden">{{rdftype}}</data>
  {% endfor %}
  {% if not inner_form %}{% csrf_token %}{% endif %}
  <fieldset>
    <legend><h2>{{legend|safe}}</h2></legend>
    {% include "spider_base/partials/base_form.html" with form=form %}
  </fieldset>
  {% if not inner_form %}
    <button class="w3-button w3-grey" style="margin-bottom: 5px">
      {{confirm}}
    </button>
  {% endif %}
<{% if not inner_form %}/form{% else %}/div{% endif %}>
