# Generated by Django 2.1.3 on 2018-11-21 07:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_filets', '0007_auto_20181016_2002'),
    ]

    operations = [
        migrations.AlterField(
            model_name='textfilet',
            name='text',
            field=models.TextField(blank=True, default=''),
        ),
    ]
